package org.globus.dynamicAccounts.client;

import org.globus.dynamicAccounts.DynamicAccountsFactoryServiceAddressingLocator;
import org.globus.dynamicAccounts.DynamicAccountsFactoryPortType;
import org.globus.dynamicAccounts.DynamicAccountsServiceAddressingLocator;
import org.globus.dynamicAccounts.DynamicAccountsPortType;
import org.globus.dynamicAccounts.types.*;
import org.globus.dynamicAccounts.impl.factory.DynamicAccountsFactoryHome;
import org.globus.dynamicAccounts.impl.service.DynamicAccountsHome;

import org.globus.wsrf.ResourceKey;
import org.globus.wsrf.impl.security.authentication.Constants;
import org.globus.wsrf.impl.security.authorization.NoAuthorization;
import org.globus.wsrf.impl.security.authorization.Authorization;
import org.globus.wsrf.impl.security.authorization.HostAuthorization;
import org.globus.wsrf.utils.AddressingUtils;

import org.apache.axis.message.addressing.EndpointReferenceType;
import org.apache.axis.client.Stub;
import org.globus.axis.util.Util;

public class DynamicAccountsClient {

    static String factoryEnd = null;
    static String serviceEnd = null;
    static String host = "localhost";
    static String port = null;
    static String cTtl = null;
    static String[] cDns = null;
    static String delEPR = null;

    protected static Object authType = Constants.GSI_TRANSPORT;
  	protected static Object authVal = Constants.SIGNATURE;
    protected static Authorization authzVal = HostAuthorization.getInstance();

    static String usage =
           "DynamicAccountsClient -h <host> -p <port> -a tt1 [dn1] [dn2]\n"
         + "                                         -d key";

    public static void main(String args[]) throws Exception {

        for (int i=0; i<args.length; i++) {


            if (args[i].equals("-h")) {
                if (i+1 >= args.length) {
                    System.err.println("-h should have host name");
                    System.exit(-1);
                }
                host = args[i+1];
            }

            if (args[i].equals("-p")) {
                if (i+1 >= args.length) {
                    System.err.println("-p should have port number");
                    System.exit(-1);
                }
                port = args[i+1];
            }

            if (args[i].equals("-a")) {
                if (i+1 >= args.length) {
                    System.err.println("-a should have ttl");
                    System.exit(-1);
                }
                cTtl = args[i+1];

                if (i+1 < args.length) {
                    cDns = new String[args.length - i - 2];
                    for (int j=i+2; j<args.length; j++) {
                        cDns[j-i-2] = args[j];
                    }
                }
            }

            if (args[i].equals("-d")) {
                if (i+1 >= args.length) {
                    System.err.println("-d should have key");
                    System.exit(-1);
                }
                delEPR = args[i+1];
            }

        }
        if (port == null) {
            if (authType.equals(Constants.GSI_TRANSPORT)) {
                port = "8443";
            } else {
                port = "8080";
            }
        }
        String protocol = "http";
        if (authType.equals(Constants.GSI_TRANSPORT)) {
            protocol = "https";
        }

        factoryEnd = protocol + "://" + host + ":" + port
            + "/wsrf/services/DynamicAccountsFactoryService";

        serviceEnd = protocol + "://" + host + ":" + port
            + "/wsrf/services/DynamicAccountsService";




        if (delEPR != null) {
            deleteAccount(delEPR);
        } else {
            String localName = createAccount(cTtl, cDns, true);
            //localName unused for now
        }

        System.exit(0);
    }

    private static void deleteAccount(String keystr)
            throws Exception {
        ResourceKey key = DynamicAccountsHome.getKey(keystr);
        EndpointReferenceType acctEPR =
            AddressingUtils.createEndpointReference(serviceEnd, key);
        DynamicAccountsServiceAddressingLocator locator1 =
            new DynamicAccountsServiceAddressingLocator();
        DynamicAccountsPortType servicePort =
            locator1.getDynamicAccountsPortTypePort(acctEPR);
        setSecurity((Stub)servicePort);
        VoidType void_t = new VoidType();
        String localName = servicePort.getLocalAccountName(void_t);
        System.out.println("local account name " + localName);
        String ownerDN = servicePort.getOwnerDN(void_t);
        System.out.println("Owner DN " + ownerDN);
        VOMSAttributes vomsattr = servicePort.getVOMSAttributes(void_t);
        if (vomsattr != null) {
            String[] attrs = vomsattr.getFqan();
            if (attrs != null) {
                for (int i = 0; i < attrs.length; i++) {
                    System.out.println("Attribute " + attrs[i]);
                }
            }
        }
        System.out.print("Deleting account ... ");
        try {
            servicePort.deleteAccount(void_t);
        } catch (Exception e) {
            System.out.println("failure:\n");
            throw e;
        }
        System.out.println("success");
    }

    private static String createAccount(String ttl,
                                        String[] policyInfo,
                                        boolean display)
        throws Exception {

        DynamicAccountsFactoryServiceAddressingLocator locator =
            new DynamicAccountsFactoryServiceAddressingLocator();
        ResourceKey key =
            DynamicAccountsFactoryHome.getResourceKey("default");
        EndpointReferenceType endpoint =
            AddressingUtils.createEndpointReference(factoryEnd, key);

        DynamicAccountsFactoryPortType port =
            locator.getDynamicAccountsFactoryPortTypePort(endpoint);
        setSecurity((Stub)port);

        AccountInformation acctInfo = new AccountInformation();
        if (ttl != null) {
            acctInfo.setTtl(new Integer(ttl));
        }
        if (policyInfo != null) {
            acctInfo.setPolicy(new AccountPolicy(policyInfo));
        }

        EndpointReferenceType acctEPR = null;
        try {
            acctEPR = port.createAccount(acctInfo);
        } catch (AccountLimitFaultType e) {
            System.out.println("Create failed: would have been duplicate account");
            //regular AccountFaultType exceptions are stacktraced to screen by
            //way of Exception being thrown
            System.exit(0);
        }

        if (display)
            System.out.println("EPR " + acctEPR);

        // Can extract localName from EPR, for now doing a port call,
        // it also ensures that things got created fine.
        DynamicAccountsServiceAddressingLocator locator1 =
            new DynamicAccountsServiceAddressingLocator();
        DynamicAccountsPortType servicePort =
            locator1.getDynamicAccountsPortTypePort(acctEPR);
        setSecurity((Stub)servicePort);

        VoidType void_t = new VoidType();

        String localName = servicePort.getLocalAccountName(void_t);
        System.out.println("localName " + localName);

        String ownerDN = servicePort.getOwnerDN(void_t);
        System.out.println("owner DN " + ownerDN);

        VOMSAttributes vomsattr = servicePort.getVOMSAttributes(void_t);
        if (vomsattr != null) {
            String[] attrs = vomsattr.getFqan();
            if (attrs != null) {
                for (int i = 0; i < attrs.length; i++) {
                    System.out.println("Attribute " + attrs[i]);
                }
            }
        } else {
            System.out.println("No attributes");

        }
        
        return localName;
    }

    private static void setSecurity(Stub stub) {
        // set security
        stub._setProperty(Constants.GSI_SEC_MSG,
                          Constants.SIGNATURE);
        stub._setProperty(Constants.AUTHORIZATION,
                          NoAuthorization.getInstance());

        if (authType.equals(Constants.GSI_TRANSPORT)) {
  	             Util.registerTransport();
  	    }
    }
}
