/*
 * This file or a portion of this file is licensed under the terms of the
 * Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without modifications,
 * you must include this notice in the file.
 */

#ifndef CONFIG_H
#define CONFIG_H

/* -------------------------------------------------------------------*/
/* Quarantine options */
/* -------------------------------------------------------------------*/

/* 
 * If DO_QUARANTINE is defined, pool accounts are not returned to the 
 * pool immediately. 
 * Instead, e.g., account file "pool001" is quarantined by hardlinking
 * "pool001.quarantined" to it. until manual intervention.  (note, when
 * there are errors deleting an account, it is always quarantined, no
 * matter if this is defined or not).
 */
#define DO_QUARANTINE

/* If QUARANTINE_BY_POOLINDEX is defined, the quarantine is reserved with
 * poolindex.poolacct.quarantine instead of poolacct.quarantine */
#define QUARANTINE_BY_POOLINDEX
 

/* -------------------------------------------------------------------*/
/* Logging options */
/* -------------------------------------------------------------------*/

/* Absolute path to logfile for this program which is different than 
 * LCMAPS logfile (see below).  See log.h for more logging and debug 
 * options.  If POOL_LOGFILE is not defined, logging is disabled.
 */

/*#define POOL_LOGFILE "/tmp/pool.log" */
#define POOL_LOGFILE "/home/globus/globus/gt4/var/log/workspace_pool.log"

/* -------------------------------------------------------------------*/
/* LCMAPS options */
/* -------------------------------------------------------------------*/

/* Paths
   Note LCMAPS install path prefix needs to be set in Makefile */

/*#define LCMAPS_LOG_FILE "/opt/glite/var/log/lcmaps_workspace.log" */
#define LCMAPS_LOG_FILE "/home/globus/globus/gt4/var/log/lcmaps_workspace.log"
/*#define LCMAPS_DB_FILE "/opt/glite/etc/lcmaps/lcmaps.db.without_gsi" */
#define LCMAPS_DB_FILE "/opt/glite/etc/lcmaps/lcmaps.db.without_gsi"

/* LCMAPS_GRIDMAPDIR needs to be set here and in $LCMAPS_DB_FILE */
#define LCMAPS_GRIDMAPDIR "/etc/grid-security/gridmapdir"


/* Other options.  Both numbers and strings need to be quoted: */
#define LCMAPS_DEBUG_LEVEL  "0"
#define LCMAPS_POLICY_NAME  "das_voms"
#define LCMAPS_LOG_STRING   "DAS poolindex"



/* -------------------------------------------------------------------*/
/* Expiry callout */
/* -------------------------------------------------------------------*/

/* 
 * Expire pool user program.  This is called *after* a lease has been
 * removed.  If quarantine is set via DO_QUARANTINE, the quarantine
 * will be in place at this point as well.  If EXPIRY_PROG fails, the
 * lease is not reinstated and the quarantine is not lifted. 
 *
 * Use absolute path.
 *
 * If not defined, no callout is made.
 *
 * The program is passed one argument, the Unix account name.
 *
 */
/* #define EXPIRY_PROG "/globus/location/bin/expire_pooluser.sh" */
#define EXPIRY_PROG "/home/globus/globus/gt4/bin/expire_pooluser.sh"

/* 
 * If defined, lcmaps_poolindex can run the EXPIRY_PROG program with any
 * effective UID (including root).
 
 * This is a convenience so that the expiry program can rely on proper 
 * inputs (which always must be checked for setuid programs, esp. if uid
 * is root).  The lcmaps_lease program checks that the arguments passed to 
 * the expiry script are valid pool accounts and valid groups.
 * 
 * If used, it is recommended that the program defined above (EXPIRY_PROG) 
 * is owned and executable exclusively by the defined UID.  Also, directories
 * 'above' EXPIRY_PROG should not be writeable by !uid.
 * 
 * If RUN_EXPIRY_SETUID is defined, the uid must be defined, like so:
 *
 * #define RUN_EXPIRY_SETUID 1004
 *
 * An example situation may be that the priviliged group is globus and
 * the UID is john.  The recommended permissions on lcmaps_poolindex in 
 * this case:
 * chmod 4750 /path/to/lcmaps_poolindex
 * chown john.globus /path/to/lcmaps_poolindex
 * 
 */

#define RUN_EXPIRY_SETUID 0

#endif

