/*
 * This file or a portion of this file is licensed under the terms of the
 * Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without modifications,
 * you must include this notice in the file.
 */

/*
 * gridmapdir_otherlink, gridmapdir_urlencode, gridmapdir_newlease and
 * gridmapdir_userid are modified functions from lcmaps_gridlist.c 
 * in org.glite.security.lcmaps-plugins-basic    
 */
 
/*                                                                                                            
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://eu-egee.org/partners/ for details on the copyright holders.
 * For license conditions see the license file or
 * http://eu-egee.org/license.html
 */

/*                                                                                                            
 * Copyright (c) 2001 EU DataGrid.                                                                             
 * For license conditions see http://www.eu-datagrid.org/license.html                                          
 *
 * Copyright (c) 2001, 2002 by 
 *     Martijn Steenbakkers <martijn@nikhef.nl>,
 *     David Groep <davidg@nikhef.nl>,
 *     NIKHEF Amsterdam, the Netherlands
 */
 

#include <utime.h>
#include <errno.h>
#include <ctype.h>
#include <dirent.h>
#include <unistd.h>
#include <sys/stat.h>
#include <pwd.h>
#include <sys/types.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <pwd.h>
#include <grp.h>
#include <malloc.h>

#include "gridmapdir.h"
#include "config.h"
#include "log.h"
#include "errors.h"
#include "lcmaps_return_poolindex_without_gsi.h"

/******************************************************************************
Function:   gridmapdir_otherlink
Description:
        find another link in MAPDIR to the same inode as firstlink
        and change the modification time of firstlink to now (so that we
        always know when this pair was last used)
        
Parameters:
        firstlink, the filename of the link we already know

Returns:
        a pointer to the other link's filename (without path) or NULL if none
        found (this is malloc'd and will need freeing)

*************************************************************************{{{ */
char 
*gridmapdir_otherlink(char *   firstlink)
{
     int            ret;
     char           *firstlinkpath, *otherlinkdup, *otherlinkpath,
                    *gridmapdir;
     struct dirent  *gridmapdirentry;
     DIR            *gridmapdirstream;
     struct stat    statbuf;
     ino_t          firstinode;
     char           *func = "gridmapdir_otherlink()";
     
     debug_msg(8,"%s",func);
     
     gridmapdir = getenv("MAPDIR");
     if (gridmapdir == NULL) {
       debug_msg(8,"%s: no gridmapdir",func);
       fprintf(stderr,"no gridmapdir\n");
       return NULL;
     }
     
     firstlinkpath = malloc(strlen(gridmapdir) + 2 + strlen(firstlink));
     sprintf(firstlinkpath, "%s/%s", gridmapdir, firstlink);     
     ret = stat(firstlinkpath, &statbuf);
     free(firstlinkpath);   
     if (ret != 0) return NULL;
     if (statbuf.st_nlink == 1) return NULL;
     if (statbuf.st_nlink > 2)
     {
         fprintf(stderr,"gridmapdir_otherlink(): too many hardlinks found (%ld) for %s\n", (long int)statbuf.st_nlink, firstlink);
         return NULL;
     }
     
     firstinode = statbuf.st_ino; /* save for comparisons */
          
//     gridmapdirstream = opendir(gridmapdir);
     if ((gridmapdirstream = opendir(gridmapdir)) == NULL)
     {
         fprintf(stderr,"gridmapdir_otherlink(): error opening directory %s: %s\n", gridmapdir, strerror(errno));
         return NULL;
     }

     if (gridmapdirstream != NULL)
     {
         while ((gridmapdirentry = readdir(gridmapdirstream)) != NULL)
         {       
                 if (strcmp(gridmapdirentry->d_name, firstlink) == 0) continue;
           
                 otherlinkpath = malloc(strlen(gridmapdir) + 2 + 
                                        strlen(gridmapdirentry->d_name));
                 sprintf(otherlinkpath, "%s/%s", gridmapdir, 
                                            gridmapdirentry->d_name);

                 ret = stat(otherlinkpath, &statbuf);     
                 if ((ret == 0) && (statbuf.st_ino == firstinode))
                 {
                      utime(otherlinkpath, (struct utimbuf *) NULL);
                      free(otherlinkpath);
                      otherlinkdup = strdup(gridmapdirentry->d_name);
                      closedir(gridmapdirstream);     
                      return otherlinkdup;
                 }
                 else free(otherlinkpath);
         }
         
         closedir(gridmapdirstream);     
     }

     return NULL;
} /* gridmapdir_otherlink() }}} */


/* __________________________________________________
   gridmapdir_unlink_index
   
   deletes association 
   _______________________________________________{{{ */
int gridmapdir_unlink_index(char *encodedglobusidp, uid_t *uid, unsigned short quarantine_flag)
{
  char *func = "gridmapdir_unlink_index()";
  char *leasedname = NULL;
  char *gridmapdir = NULL;
  char *quarantinepath = NULL; 
  char *leasepath = NULL;
  char *quarantinesuffix = QUARANTINE_SUFFIX;
  char *encodedglobusidpath = NULL;
  struct stat    statbuf;
  int ret;
  struct passwd *pwd;
     
  debug_msg(8,"%s",func);
     
  if (encodedglobusidp == NULL) {
    debug_msg(1,"%s problem, encodedglobusidp is null",func);
    ret = BADARG;
    goto free;
  } 
  
  if (uid == NULL) {
    debug_msg(1,"%s problem, uid * is null",func);
    ret = GENERAL_FAILURE;
    goto free;
  }
  
  leasedname = gridmapdir_otherlink(encodedglobusidp);
  
  if (!leasedname) {
    debug_msg(3,"%s can't delete: no association for: %s",func,encodedglobusidp);
    fprintf(stderr,"can't delete: no association for: %s\n",encodedglobusidp);
    ret = GRIDMAPDIR_NO_ASSOCIATION;
    goto free;
  }
  
  gridmapdir = getenv("MAPDIR");
  if (gridmapdir == NULL) { 
    /* gridmapdir_otherlink() returns null if no MAPDIR, so.. */
    debug_msg(1,"%s: impossible, no MAPDIR here?",func);
    goto fail;
  }
  
  leasepath = (char *) malloc(strlen(gridmapdir) + 2 + strlen(leasedname));
  if (!leasepath) {
    debug_msg(1,"%s problem, malloc",func);
    goto fail;
  }
  sprintf(leasepath, "%s/%s", gridmapdir, leasedname);
  debug_msg(8,"%s: account = %s",func,leasepath);
  
  /* first link account file to a quarantine file */
  if (quarantine_flag & QUARANTINE) {
    debug_msg(3,"%s: attempting to quarantine before unlinking",func);
    
    /* Leaving option to QUARANTINE_BY_POOLINDEX but it is not recommended.
     * If a user leases another account, then upon that account's deletion
     * it is not possible to secure another quarantine!
     */
#ifdef QUARANTINE_BY_POOLINDEX
    quarantinepath = malloc(strlen(gridmapdir) + strlen(encodedglobusidp) + strlen(leasedname) + 4 + strlen(quarantinesuffix));
#else
    quarantinepath = malloc(strlen(leasepath) + 2 + strlen(quarantinesuffix));
#endif
    
    if (!quarantinepath) {
      debug_msg(1,"%s problem, malloc",func);
      goto fail;
    }
    
#ifdef QUARANTINE_BY_POOLINDEX
    sprintf(quarantinepath, "%s/%s.%s%s", gridmapdir, encodedglobusidp, leasedname, quarantinesuffix);
#else
    sprintf(quarantinepath, "%s%s", leasepath, quarantinesuffix);
#endif
    debug_msg(8,"%s: quarantine to %s",func,quarantinepath);
    
    ret = stat(quarantinepath, &statbuf);
    if (ret == 0) {
      debug_msg(1,"%s: cannot quarantine, file already exists: %s",func,quarantinepath);
      /* TODO: here, override inconsistency (touch file) or just continue */
      goto fail;
      
    }
    
    ret = link(leasepath,quarantinepath);
    if (ret) {
      debug_msg(1,"%s: cannot link %s and %s", func, leasepath, quarantinepath);
      goto fail;
    }
  }
  
  encodedglobusidpath = (char *) malloc(strlen(gridmapdir) + 2 + strlen(encodedglobusidp));
  if (!encodedglobusidpath) {
    debug_msg(1,"%s problem, malloc",func);
    goto fail;
  }
  sprintf(encodedglobusidpath, "%s/%s", gridmapdir, encodedglobusidp);
  
  debug_msg(8,"%s: encodedglobusidpath = %s",func,encodedglobusidpath);
  
  ret = unlink(encodedglobusidpath);
  if (ret) { 
    /* todo, parse errno, this is an odd inconsistency */
    debug_msg(1,"%s: cannot unlink %s", func, encodedglobusidpath);
    goto fail;
  }
  
  if ((pwd = getpwnam(leasedname)) == NULL)
  {
    debug_msg(1,"%s: very unlikely.. username %s not found?", func, leasedname);
    goto fail;
  }

  *uid = pwd->pw_uid;
  endpwent();
  
  debug_msg(8,"%s: unlinked successfully, found uid = %d", func, *uid);
  
  ret = 0;
  goto free;
  fail:
    ret = 1;
  free:
    if (leasedname) free(leasedname);
    if (leasepath) free(leasepath);
    if (quarantinepath) free(quarantinepath);
    if (encodedglobusidpath) free(encodedglobusidpath);
    return ret;
} /* gridmapdir_unlink_dn() }}} */


/* __________________________________________________
   call LCMAPS
   _______________________________________________{{{ */
int lcmaps_poolindex(char *dn, char *fqan_list[], int nfqan, lcmaps_account_info_t *lcmaps_account, char **gridmapdir)
{
  int ret;
  char * lcmaps_suffix = LCMAPS_LOG_STRING;
  char * lcmaps_log_string = NULL;
  char * time_prefix = get_time();
  
  if (time_prefix == NULL) {
    return GENERAL_FAILURE;
  }
  
  lcmaps_log_string =  (char *) malloc ((strlen(lcmaps_suffix) + strlen(time_prefix) + strlen(" : ")) * sizeof(char) + 1);
  if (lcmaps_log_string == NULL) {
    return GENERAL_FAILURE;
  }
  
  strcpy(lcmaps_log_string, time_prefix);
  strcat(lcmaps_log_string, " : ");
  strcat(lcmaps_log_string, lcmaps_suffix);
  
  setenv("LCMAPS_LOG_FILE", LCMAPS_LOG_FILE , 1);
//    setenv("LCMAPS_LOG_FILE", "/dev/stderr", 1);
  setenv("LCMAPS_DB_FILE", LCMAPS_DB_FILE , 1);
  setenv("LCMAPS_DEBUG_LEVEL", LCMAPS_DEBUG_LEVEL , 1);
  setenv("LCMAPS_POLICY_NAME", LCMAPS_POLICY_NAME , 1);
  setenv("LCMAPS_LOG_STRING", lcmaps_log_string , 1);
  
  if (lcmaps_log_string) free (lcmaps_log_string);
  if (time_prefix) free (time_prefix);

  ret = lcmaps_account_info_init(lcmaps_account);
  debug_msg(5, "Result of lcmaps_account_info_init(): %d", ret);
  
  /* not that this is completely accurate, but if 
     lcmaps_account_info_init() fails it will be POOL_INDEX_FAILURE. 
     If it succeeds but lcmaps_return_poolindex_without_gsi fails, return
     GRIDMAPDIR_NO_ASSOCIATION
   */
  if (ret) {
    return POOL_INDEX_FAILURE;
  }

  ret = lcmaps_return_poolindex_without_gsi(
      dn,
      fqan_list,
      nfqan,
      lcmaps_account
  );
  debug_msg(5, "Result of lcmaps_return_poolindex_without_gsi(): %d", ret);

  if (ret == 0)
  {
    int i = 0;
    debug_msg(5, "lcmaps_account.poolindex: %s", lcmaps_account->poolindex);
    debug_msg(5, "lcmaps_account.uid: %d", (int) lcmaps_account->uid);
    debug_msg(5, "lcmaps_account.npgid: %d", (int) lcmaps_account->npgid);
    for (i = 0; i < lcmaps_account->npgid; i++)
        debug_msg(5, "lcmaps_account.pgid_list[%d]: %d", i, lcmaps_account->pgid_list[i]);
    debug_msg(5, "lcmaps_account.nsgid: %d", (int) lcmaps_account->nsgid);
    for (i = 0; i < lcmaps_account->nsgid; i++)
        debug_msg(5, "lcmaps_account.sgid_list[%d]: %d", i, lcmaps_account->sgid_list[i]);
    
    *gridmapdir = getenv("MAPDIR");
    if (*gridmapdir == NULL) {
      fprintf(stderr, "gridmapdir not set\n");
      log_msg("gridmapdir not set by LCMAPS\n");
      return ENVIRONMENT;
    }
    
    return SUCCESS;
  } else {

    ret = lcmaps_account_info_clean(lcmaps_account);
    fprintf(stderr, "Result of lcmaps_account_info_clean(): %d\n", ret);
    return GRIDMAPDIR_NO_ASSOCIATION;
  }
    
} /* lcmaps_poolindex() }}} */


