#!/bin/bash

# READ CAREFULLY

# Site specific pool expiration script.  Called after lease is expired
# by the unlinking in the gridmapdir.  Also note this is called whether
# or not quarantine is compiled in to the poolaccts setuid, that is
# part of the gridmapdir functionality and handled by the setuid program.

# This script is called from the C setuid program with suid/euid 0
# Be careful, if you need to change this behavior, change/add setuid/seteuid
# call after if (HANDLE_POOL_EXPIRE == 0) in poolaccts.c and recompile.

# Below is a sample expiry/cleaning.  To ensure correct behavior experiment 
# with a fake account before deploying!!

# 1. user's processes are killed. 
# 2. user's files in ~ and /tmp are deleted
# 3. account's primary group is set to the default


#################
# Configuration #
#################

POOL_PREFIX="gpool"   # only works with one pool for now
POOL_PREFIX_STRLEN=5  # number of letters in your pool prefix

DEFAULT_GROUP="users"      # revert group membership to this default group

ETC_PASSWD_FIELD=6    # which field is the user's homedir in /etc/passwd


RM_PROG="/bin/rm"
RMDIR_PROG="/bin/rmdir"
KILL_PROG="/bin/kill"
CUT_PROG="/bin/cut"
ECHO_PROG="/bin/echo"
GREP_PROG="/bin/grep"
FIND_PROG="/usr/bin/find"
USERMOD_PROG="/usr/sbin/usermod"


#############
# Functions #
#############

killuserprocs () {
  KILLED=0
  USERPROCS=`$FIND_PROG /proc/ -user $1 -maxdepth 1 | cut -b 7-11`
  for i in $USERPROCS; do
    $ECHO_PROG "killing $i" 1>&2
    $KILL_PROG -9 $i
    if [ $? -eq 0 ]; then
      (( KILLED += 1 )) 
    fi
  done
  return $KILLED
}


#######
# Run #
#######

if [ $1 ]; then
  
  if [ $1 = "root" ]; then
    $ECHO_PROG "will not run with argument 'root'" 1>&2
    exit 1
  fi
  
  # make sure argument starts with a pool account prefix
  USERPREF=`echo $1 | $CUT_PROG -b 1-$POOL_PREFIX_STRLEN`
  if [ $USERPREF != $POOL_PREFIX ]; then
    $ECHO_PROG "user not a pool user" 1>&2
    exit 1
  fi
  
  # kill procs, 3 times to be thorough
  for i in 1 2 3; do
    killuserprocs $1
    $ECHO_PROG "procs killed: $?" 1>&2
  done
  
  USERPROCS=`$FIND_PROG /proc/ -user $1 -maxdepth 1 | $CUT_PROG -b 7-11`
  FIRSTPROC=`$ECHO_PROG "$USERPROCS" | cut -b 1-5`
  if [ $FIRSTPROC ]; then
    $ECHO_PROG "could not kill all user processes!" 1>&2
    # error??
  fi
    
  HOMEDIR=`$GREP_PROG $1 /etc/passwd | cut -d : -f $ETC_PASSWD_FIELD`
  
  # Delete all files owned by user
  $FIND_PROG $HOMEDIR /tmp -user $1 ! -type d -exec $RM_PROG -f {} \;
  
  # Delete all directories owned by user (except home dir)
  $FIND_PROG $HOMEDIR /tmp -mindepth 1 -depth -user $1 -type d -exec $RMDIR_PROG {} \;
  
  # error on files we cannot delete?
  
  
  # set primary group
  $USERMOD_PROG -g $DEFAULT_GROUP $1
  if [ $? -ne 0 ]; then
    $ECHO_PROG "cannot change user: $1 to default group: $DEFAULT_GROUP" 1>&2
    exit 1
  fi
  
  exit 0
  
else 
  $ECHO_PROG "no argument" 1>&2
  exit 1
fi
  
  
