/*
 * This file or a portion of this file is licensed under the terms of the
 * Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without modifications,
 * you must include this notice in the file.
 */
package org.globus.dynamicAccounts.impl.factory;

import org.globus.wsrf.impl.ResourceHomeImpl;
import org.globus.wsrf.impl.SimpleResourceKey;

import java.util.StringTokenizer;

import javax.xml.namespace.QName;

import org.globus.dynamicAccounts.DynamicAccountsConstants;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class DynamicAccountsFactoryHome extends ResourceHomeImpl {

    static Log logger = 
        LogFactory.getLog(DynamicAccountsFactoryHome.class.getName());    

    String creationMech = null;
    String defaultTtl = null;
    String host = null;
    String authzInfoFile = "authzInfoFile";

    static final QName RESOURCE_KEY_QNAME =
        new QName(DynamicAccountsConstants.NS, "ResourceID");

    // Create a resource per creation mechanism configured in jndi
    // config. Each resource key is created using the creation
    // mechanism name.

    // For now, we just use "default" for creation mechanism.
    // The implementation is decided on by the site administrator at deploy time. 
    public synchronized void initialize() throws Exception {
        this.keyTypeName = RESOURCE_KEY_QNAME;
        super.initialize();
        logger.debug("Creating resources " + this.creationMech);

        if ((this.creationMech == null) || (this.defaultTtl == null)
            || (this.authzInfoFile == null)) {
            String err = "Home/Ttl/AuthzInfo configuration was not set";
            logger.debug(err);
            throw new Exception(err);
        }
        StringTokenizer st = new StringTokenizer(this.creationMech);
        StringTokenizer ttlStrTok = new StringTokenizer(this.defaultTtl);
        StringTokenizer authzInfoTok = new StringTokenizer(this.authzInfoFile);

        while (st.hasMoreTokens()) {
            String str = st.nextToken();
            logger.debug("Creating factory resource for " + str);
            SimpleResourceKey key = getResourceKey(str);
            DynamicAccountsFactoryResource factoryResource =
                (DynamicAccountsFactoryResource)createNewInstance();
            Integer ttl = null;
            String authzInfoStr = null;
            if (ttlStrTok.hasMoreTokens()) {
                String ttlStr = ttlStrTok.nextToken();
                ttl = new Integer(ttlStr);
            }
            if (authzInfoTok.hasMoreTokens()) {
                authzInfoStr = authzInfoTok.nextToken();
            }
            factoryResource.initialize(key, ttl, this.host, authzInfoStr);
            logger.debug("Adding key " +  key);
            this.add(key, factoryResource);
        }
    }

    public static final SimpleResourceKey getResourceKey(String keyValue) {
        return new SimpleResourceKey(RESOURCE_KEY_QNAME, keyValue);
    }

    public void setCreationMechanism(String _creationMech) {
        this.creationMech = _creationMech;
    }

    public String getCreationMechanism() {
        return this.creationMech;
    }

    public void setDefaultTtl(String ttl) {
        this.defaultTtl = ttl;
    }

    public String getDefaultTtl() {
        return this.defaultTtl;
    }

    public void setHost(String _host) {
        this.host = _host;
    }

    public String getHost() {
        return this.host;
    }

    public void setAuthzInfoFilename(String _authzInfoFile) {
        this.authzInfoFile = _authzInfoFile;
    }

    public String getAuthzInfoFilename() {
        return this.authzInfoFile;
    }
}

