/*
 * This file or a portion of this file is licensed under the terms of the
 * Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without modifications,
 * you must include this notice in the file.
 */
package org.globus.dynamicAccounts.impl.service;

import org.globus.dynamicAccounts.DynamicAccountsConstants;
import org.globus.dynamicAccounts.DynamicAccountsException;
import org.globus.dynamicAccounts.LimitException;

import org.globus.dynamicAccounts.types.AccountInformation;

import org.globus.wsrf.ResourceKey;
import org.globus.wsrf.ResourceException;
import org.globus.wsrf.Resource;
import org.globus.wsrf.NoSuchResourceException;
import org.globus.wsrf.config.ContainerConfig;

import org.globus.wsrf.impl.ResourceHomeImpl;
import org.globus.wsrf.impl.SimpleResourceKey;

import javax.xml.namespace.QName;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class DynamicAccountsHome extends ResourceHomeImpl {

    static final QName RESOURCE_KEY_QNAME =
        new QName(DynamicAccountsConstants.NS, "AccountKey");

    static Log logger = 
        LogFactory.getLog(DynamicAccountsHome.class.getName());    
    
    String globusLocation = ContainerConfig.getGlobusLocation();

    public synchronized void initialize() throws Exception {
        this.keyTypeName = RESOURCE_KEY_QNAME;
        super.initialize();
    }

    // This method creates a resource per account and the creation of
    // the actual account should be based on the creationMechanism
    // that is passed as parameter. For now, ignoring it.
    public synchronized ResourceKey create(AccountInformation accountInfo,
                                           String creationMechanism, 
                                           String callerDN,
                                           String callerUsername,
                                           int defaultTtl)
        throws DynamicAccountsException, LimitException {
     
        DynamicAccountsResource resource = null;
        try {
            resource = (DynamicAccountsResource)createNewInstance();
            resource.initialize(creationMechanism,
                    callerDN,
                    callerUsername,
                    this.globusLocation,
                    defaultTtl);
        } catch (ResourceException exp) {
            logger.error(exp);
            throw new DynamicAccountsException(exp);
        }
        
        // This is used as part of key, addAuthz code relies on this behavior
        String username = resource.createAccount(accountInfo);

        if (username == null) {
            throw new DynamicAccountsException("null mapping returned");
        }

        // locking: Instead of relying on backend to say if pool lease
        // is new or not (which would be ideal), see if a resource (account) is
        // in the list and reject if there already.  Adduser impl resources will
        // pass through because user will (should) always be new.
        ResourceKey key = getKey(username);
        logger.debug("new key, " + key.getName());
        return addToHash(resource,key);
    }
    
    public static ResourceKey getKey(String localName) {
        return new SimpleResourceKey(RESOURCE_KEY_QNAME, localName);
    }

    private synchronized ResourceKey addToHash(DynamicAccountsResource resource,
                                               ResourceKey key)
            throws LimitException, DynamicAccountsException {
        try {
            try {
                Resource rsrc = this.find(key);
            } catch (NoSuchResourceException e) {
                this.add(key, resource);
                return key;
            }
            logger.debug("found equal resource");
            throw new LimitException("will not create duplicate account");
        } catch (ResourceException e) {
            throw new DynamicAccountsException(e);
        }
    }

}
