/*
 * This file or a portion of this file is licensed under the terms of the
 * Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without modifications,
 * you must include this notice in the file.
 */
package org.globus.dynamicAccounts.impl.service;

import org.globus.dynamicAccounts.types.VoidType;
import org.globus.dynamicAccounts.types.AccountFaultType;
import org.globus.dynamicAccounts.types.VOMSAttributes;

import org.globus.dynamicAccounts.DynamicAccountsUtil;
import org.globus.dynamicAccounts.DynamicAccountsException;

import org.globus.wsrf.ResourceContext;
import org.globus.wsrf.ResourceException;
import org.globus.wsrf.ResourceContextException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class DynamicAccountsService {

    static Log logger = 
        LogFactory.getLog(DynamicAccountsService.class.getName());    

    public VoidType deleteAccount(VoidType voidType) throws AccountFaultType {
        logger.debug("Delete account ");
        try {
            DynamicAccountsResource resource = getResource();
            resource.deleteAccount();
        } catch (DynamicAccountsException exp) {
            String err = "Error deleting account";
            logger.error(err, exp);
            throw DynamicAccountsUtil.makeFault(err, exp);
        }
        return new VoidType();
    }

    public VoidType addAuthzIdentity(String dn) throws AccountFaultType {
        DynamicAccountsResource resource = null;
        try {
            resource = getResource();
            resource.addAuthzIdentity(dn);
        } catch (DynamicAccountsException exp) {
            String err = "Error adding authz identity";
            logger.error(err, exp);
            throw DynamicAccountsUtil.makeFault(err, exp);
        }
        return new VoidType();
    }

    public VoidType removeAuthzIdentity(String dn) throws AccountFaultType {
        DynamicAccountsResource resource = null;
        try {
            resource = getResource();
            resource.removeAuthzIdentity(dn);
        } catch (DynamicAccountsException exp) {
            String err = "Error removing authz identity";
            logger.error(err, exp);
            throw DynamicAccountsUtil.makeFault(err, exp);
        }
        return new VoidType();
    }

    public String getLocalAccountName(VoidType voidType)
        throws AccountFaultType {
        DynamicAccountsResource resource = null;
        try {
            resource = getResource();
            return resource.getLocalAccountName();
        } catch (DynamicAccountsException exp) {
            String err = "Error getting local account name";
            logger.error(err, exp);
            throw DynamicAccountsUtil.makeFault(err, exp);
        }
    }

    public String getOwnerDN(VoidType voidType)
        throws AccountFaultType {
        DynamicAccountsResource resource = null;
        try {
            resource = getResource();
            return resource.getOwnerDN();
        } catch (DynamicAccountsException exp) {
            String err = "Error getting account owner DN";
            logger.error(err, exp);
            throw DynamicAccountsUtil.makeFault(err, exp);
        }
    }

    public VOMSAttributes getVOMSAttributes(VoidType voidType)
        throws AccountFaultType {
        DynamicAccountsResource resource = null;
        try {
            resource = getResource();
            return resource.getVOMSAttributes();
        } catch (DynamicAccountsException exp) {
            String err = "Error getting secondary groups";
            logger.error(err, exp);
            throw DynamicAccountsUtil.makeFault(err, exp);
        }
    }

    private DynamicAccountsResource getResource()
        throws DynamicAccountsException {
        try {
            ResourceContext context = ResourceContext.getResourceContext();
            return (DynamicAccountsResource)context.getResource();
        } catch (ResourceContextException exp) {
            String err = "Error getting resource";
            logger.error(err, exp);
            throw new DynamicAccountsException(err, exp);
        } catch (ResourceException exp) {
            String err = "Error getting factory resource";
            logger.error(err, exp);
            throw new DynamicAccountsException(err, exp);
        }
    }
}
