package org.globus.dynamicAccounts.impl.service;

import org.globus.dynamicAccounts.DynamicAccountsException;
import org.globus.dynamicAccounts.DynamicAccountsUtil;
import org.globus.dynamicAccounts.ReturnException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.io.File;
import java.util.StringTokenizer;

public class PoolAccountsResource extends DynamicAccountsResourceImpl {

    static Log logger =
        LogFactory.getLog(DynamicAccountsResourceImpl.class.getName());

    /* set in initialize() */
    protected String executable = null;

    protected final static String BASE_COMMAND = "bin" + File.separator
        + "poolaccts_setuid";
    protected final static String POOLINDEX_COMMAND = "-return_poolindex";
    protected final static String DELETE_COMMAND = "-delete_dn";


    //overriding to set executable
    public void initialize(String _creationMechanism,
                       String _callerDN,
                       String _callerUserName,
                       String globusLocation,
                       int defaultTtl)
        throws DynamicAccountsException {

        super.initialize(_creationMechanism, _callerDN, _callerUserName, globusLocation,defaultTtl);

        if (globusLocation.endsWith(File.separator)) {
            executable = globusLocation + BASE_COMMAND;
        } else {
            executable = globusLocation + File.separator + BASE_COMMAND;
        }
        logger.debug("Executable " + executable);

    }


    /*
     * BEGIN setuid interface related methods
     */

    protected void createAccountUsingMech() throws DynamicAccountsException {

        String command[] = null;
        if (this.primaryAttr != null) {
            command =  new String[4];
            command[0] = executable;
            command[1] = POOLINDEX_COMMAND;
            command[2] = this.callerDN;
            command[3] = this.primaryAttr;
        } else {
            command =  new String[3];
            command[0] = executable;
            command[1] = POOLINDEX_COMMAND;
            command[2] = this.callerDN;
        }

        String returnString = null;
        try {
            returnString = DynamicAccountsUtil.runCommand(command);
        } catch (ReturnException e) {
            translateReturnValue(e);
        }

        logger.debug("Return string " + returnString);
        StringTokenizer st = new StringTokenizer(returnString,":");
        if (st.hasMoreTokens()) {
            this.localName = st.nextToken();
            if (st.hasMoreTokens()) {
                this.primaryGroup = st.nextToken();
            } else {
                throw new DynamicAccountsException("no group returned from pool impl");
            }
        } else {
            throw new DynamicAccountsException("no account returned from pool impl");
        }
        logger.debug("Local name " + this.localName);
        logger.debug("Primary group " + this.primaryGroup);
    }

    protected void deleteAccountUsingMech() throws DynamicAccountsException {
        String command[] =  new String[3];
        command[0] = executable;
        command[1] = DELETE_COMMAND;
        command[2] = this.callerDN;
        try {
            DynamicAccountsUtil.runCommand(command);
        } catch (ReturnException e) {
            translateReturnValue(e);
        }
    }

    private void translateReturnValue(ReturnException e) throws DynamicAccountsException {
        /* #define GENERAL_FAILURE 255 */
        if (e.retval == 255) {
            if (e.stderr != null) {
            throw new DynamicAccountsException("failure running pool" +
                    " account setuid\nSTDERR: " + e.stderr);
            } else {
            throw new DynamicAccountsException("failure running pool" +
                        " account setuid");
            }
        }
        /* #define ENVIRONMENT 1 */
        if (e.retval == 1) {
            if (e.stderr != null) {
            throw new DynamicAccountsException("pool account setuid" +
                    " setup problem\nSTDERR: " + e.stderr);
            } else {
            throw new DynamicAccountsException("pool account setuid" +
                    " setup problem");
            }
        }
        /* #define SYNTAX 2 */
        if (e.retval == 2) {
            if (e.stderr != null) {
            throw new DynamicAccountsException("pool account service" +
                    " setup problem\nSTDERR: " + e.stderr);
            } else {
            throw new DynamicAccountsException("pool account service" +
                    "setup problem");
            }
        }
        /* #define BADARG 3 */
        if (e.retval == 3) {
            if (e.stderr != null) {
            throw new DynamicAccountsException("problem with parameters, STDERR: " + e.stderr);
            } else {
            throw new DynamicAccountsException("problem with parameters");
            }
        }
        /* #define POOL_INDEX_FAILURE 4 */
        if (e.retval == 4) {
            if (e.stderr != null) {
            throw new DynamicAccountsException("problem with mappings, STDERR: " + e.stderr);
            } else {
            throw new DynamicAccountsException("problem with mappings");
            }
        }
        /* #define NO_POOL_FOR_USER 5 */
        if (e.retval == 5) {
            if (e.stderr != null) {
            throw new DynamicAccountsException("found user for DN, but no pool account, STDERR: " + e.stderr);
            } else {
            throw new DynamicAccountsException("found user for DN, but no pool account,");
            }
        }
        /* #define NO_USER 6 */
        if (e.retval == 6) {
            if (e.stderr != null) {
            throw new DynamicAccountsException("no mapping for this DN, STDERR: " + e.stderr);
            } else {
            throw new DynamicAccountsException("no mapping for this DN");
            }
        }
        /* #define POOL_FULL 7 */
        if (e.retval == 7) {
            if (e.stderr != null) {
            throw new DynamicAccountsException("pool is exhausted, STDERR: " + e.stderr);
            } else {
            throw new DynamicAccountsException("pool is exhausted");
            }
        }
        /* #define GRIDMAPDIR_NO_ASSOCIATION 8 */
        if (e.retval == 8) {
            if (e.stderr != null) {
            throw new DynamicAccountsException("pooling environment problem, STDERR: " + e.stderr);
            } else {
            throw new DynamicAccountsException("pooling environment problem");
            }
        }


        /*****
         * EXPIRY_PROG_FAILURE & EXPIRY_PROG_CHECK are both errors that happen
         * after a lease has been unleased and quarantined.  Perhaps don't throw
         * exception because this resource, which represents the lease, must succeed
         * in its destruction?
         */

        /* #define EXPIRY_PROG_FAILURE 9 */
        if (e.retval == 9) {
            if (e.stderr != null) {
            throw new DynamicAccountsException("expire account program failure, STDERR: " + e.stderr);
            } else {
            throw new DynamicAccountsException("expire account program failure");
            }
        }

        /* #define EXPIRY_PROG_CHECK 10 */
        if (e.retval == 10) {
            if (e.stderr != null) {
            throw new DynamicAccountsException("expire account environment problem, STDERR: " + e.stderr);
            } else {
            throw new DynamicAccountsException("expire account environment problem");
            }
        }

        /* #define NO_GROUP 11 */
        if (e.retval == 11) {
            if (e.stderr != null) {
            throw new DynamicAccountsException("no group mapping for attribute, STDERR: " + e.stderr);
            } else {
            throw new DynamicAccountsException("no group mapping for attribute");
            }
        }

        /* others? */
        if (e.stderr != null) {
        throw new DynamicAccountsException("pooling problem, implementation " +
                "returns error #" + e.retval + ", STDERR: " + e.stderr);
        } else {
        throw new DynamicAccountsException("pooling problem, implementation" +
                "returns error #" + e.retval);
        }

    }

    /*
     * END setuid interface related methods
     */

}
