/*
 * This file or a portion of this file is licensed under the terms of the
 * Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without modifications,
 * you must include this notice in the file.
 */
package org.globus.dynamicAccounts.impl.service;

import java.io.File;

import org.globus.dynamicAccounts.DynamicAccountsUtil;
import org.globus.dynamicAccounts.DynamicAccountsException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class UnixAccountsResource extends DynamicAccountsResourceImpl {


    final static String BASE_COMMAND = "bin" + File.separator
        + "adduser_setuid";
    final static String ADDUSER_COMMAND = "-adduser";
    final static String GETQUOTA_COMMAND = "-getquota";
    final static String SETQUOTA_COMMAND = "-setquota";
    final static String DELUSER_COMMAND = "-deluser";
    String executable = null;

    static Log logger =
        LogFactory.getLog(UnixAccountsResource.class.getName());

    public void initialize(String _creationMechanism,
                           String _callerDN,
                           String _callerUserName,
                           String globusLocation,
                           int defaulTtl) {

        if (globusLocation.endsWith(File.separator)) {
            executable = globusLocation + BASE_COMMAND;
        } else {
            executable = globusLocation + File.separator + BASE_COMMAND;
        }
        logger.error("Executable " + executable);
    }

    protected void createAccountUsingMech() throws DynamicAccountsException {

        // Create actual account
        String command[] =  new String[4];
        command[0] = executable;
        command[1] = ADDUSER_COMMAND;
        command[2] = this.callerDN;
        command[3] = this.callerUsername;

        String err = "Error creating new account ";
        String returnString = DynamicAccountsUtil.runCommand(command, err);
	    logger.debug("Return string " + returnString);
	    this.localName = returnString.substring(0, returnString.indexOf(":"));
	    logger.debug("Local name " + this.localName);
        this.primaryGroup = returnString.substring(returnString.indexOf(":"));
	    logger.debug("Primary group " + this.primaryGroup);
    }

    protected void deleteAccountUsingMech() throws DynamicAccountsException {
        String command[] =  new String[3];
        command[0] = executable;
        command[1] = DELUSER_COMMAND;
        command[2] = this.localName;
        String err = "Error deleting account " + this.localName;
        DynamicAccountsUtil.runCommand(command, err);
    }

    protected void constructGroupMapInput() throws DynamicAccountsException {
        //nothing to do
    }

}
